(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7741,        258]*)
(*NotebookOutlinePosition[      8599,        288]*)
(*  CellTagsIndexPosition[      8555,        284]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\

(The functions used in this notebook require the C++ NCGB module.)\
\>", 
  "Text",
  Editable->False,
  FontSize->14],

Cell[CellGroupData[{

Cell["\<\

       The Partially Prescribed Inverse Problem
\
\>", "H2",
  Editable->False,
  CellFrame->{{0, 0}, {2, 0}},
  TextAlignment->Center,
  TextJustification->0.5],

Cell["\<\

This is a type of problem known as a matrix completion problem. This \
particular one was  suggested by Hugo Woerdeman.   We are grateful to him for \
discussions.
\
\>", "Text",
  Editable->False],

Cell[TextData[StyleBox[
"Problem:\n     \tGiven matrices a, b, c,  and  d,    we  wish to determine \
under what conditions there exists matrices  x,  y,  z, and w such that the \
block matrices",
  FontWeight->"Bold"]], "Text"],

Cell[TextData[{
  "     \n     ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", GridBox[{
              {"a", "x"},
              {"y", "b"}
              }], ")"}], TraditionalForm]]],
  "                       ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", GridBox[{
              {"w", "c"},
              {"d", "z"}
              }], ")"}], TraditionalForm]]]
}], "Text",
  Editable->False,
  TextAlignment->Center],

Cell[TextData[{
  "\n",
  StyleBox[
  "are inverses of each other.     Also, we wish to find formulas for x,  y, \
z,  and  w.",
    FontWeight->"Bold"],
  "\n\n    This problem was solved in a paper by  W.W. Barrett, C.R. Johnson, \
M. E. Lundquist  and H. Woerderman  ",
  StyleBox["[BJLW]",
    FontWeight->"Bold"],
  " where  they showed it splits into several cases depending upon which of \
a, b, c and d are invertible. In our example, ",
  StyleBox["we assume that a, b, c and d  are invertible",
    FontWeight->"Bold"],
  " and discover the result which they obtain in this case."
}], "Text",
  CellFrame->{{0, 0}, {2, 0}}],

Cell["\<\
First we set all of our variables to be noncommutative and set up \
the relations which make matrices a, b, c, and d invertible.  (Inverses in \
this particular problem are taken to be two sided.)   Strong invertibility \
relations help when one is trying to get an idea of the solution of the \
problem.
\
\>", "Text",
  Editable->False,
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\n\ \ \ \ \ \ \ \ \ \ \ \ \ 
    SetNonCommutative[a, b, c, d, w, x, y, z, \ Inv[a], \ \ Inv[b], \ 
      Inv[c]\ , Inv[d]\ \ ]\n\)\)], "Input"],

Cell[BoxData[
    \({False, False, False, False, False, False, False, False, False, False, 
      False, False}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Then we define the relations we are interested in.   The  two relations ",
  StyleBox["oneway, otherway",
    FontWeight->"Bold"],
  "  set our block matrices as inverses of each other.   The relations",
  StyleBox[" inverses",
    FontWeight->"Bold"],
  "\ninvoke the assumption that ",
  StyleBox["a,b,c,",
    FontWeight->"Bold"],
  " and ",
  StyleBox["d",
    FontWeight->"Bold"],
  " are invertible by defining their inverses."
}], "Text",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(first\  = \ {{a, x}, {y, b}}\), 
    \(second\  = \ {{w, c}, {d, z}}\)}], "Input"],

Cell[BoxData[
    \({{a, x}, {y, b}}\)], "Output"],

Cell[BoxData[
    \({{w, c}, {d, z}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\noneway\  = MatMult[first, second]\  - \ IdentityMatrix[2]\), 
    \(otherway\  = \ \ MatMult[second, first] - \ \ IdentityMatrix[2]\)}], 
  "Input"],

Cell[BoxData[
    \({{\(-1\) + a**w + x**d, a**c + x**z}, {b**d + y**w, 
        \(-1\) + b**z + y**c}}\)], "Output"],

Cell[BoxData[
    \({{\(-1\) + c**y + w**a, c**b + w**x}, {d**a + z**y, 
        \(-1\) + d**x + z**b}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\ninverses\  = 
      \ {\(-1\)\  + \ a\ **\ Inv[a], \ \(-1\)\  + \ Inv[a]\ **\ a, \n\ \ \ \ 
        \(-1\)\  + \ b\ **\ Inv[b], \ \(-1\)\  + \ Inv[b]\ **\ b, \n\ \ \ \ 
        \(-1\)\  + \ c\ **\ Inv[c], \ \(-1\)\  + \ Inv[c]\ **\ c, \n\ \ \ \ \ 
        \(-1\)\  + \ d\ **\ Inv[d], \ \(-1\)\  + \ Inv[d]\ **\ d\ \n\t}\n\), 
    \(allRelations\  = 
      Join[\ Flatten[{\ oneway, \ otherway\ \ }], inverses]\)}], "Input"],

Cell[BoxData[
    \({\(-1\) + a**w + x**d, a**c + x**z, b**d + y**w, \(-1\) + b**z + y**c, 
      \(-1\) + c**y + w**a, c**b + w**x, d**a + z**y, \(-1\) + d**x + z**b}
      \)], "Output"],

Cell[BoxData[
    \({\(-1\) + a**Inv[a], \(-1\) + Inv[a]**a, \(-1\) + b**Inv[b], 
      \(-1\) + Inv[b]**b, \(-1\) + c**Inv[c], \(-1\) + Inv[c]**c, 
      \(-1\) + d**Inv[d], \(-1\) + Inv[d]**d}\)], "Output"],

Cell[BoxData[
    \({\(-1\) + a**w + x**d, a**c + x**z, b**d + y**w, \(-1\) + b**z + y**c, 
      \(-1\) + c**y + w**a, c**b + w**x, d**a + z**y, \(-1\) + d**x + z**b, 
      \(-1\) + a**Inv[a], \(-1\) + Inv[a]**a, \(-1\) + b**Inv[b], 
      \(-1\) + Inv[b]**b, \(-1\) + c**Inv[c], \(-1\) + Inv[c]**c, 
      \(-1\) + d**Inv[d], \(-1\) + Inv[d]**d}\)], "Output"]
}, Open  ]],

Cell["\<\
Specify to NCAlgebra which variables are known and which are \
unknown. To GB fans this sets the monomial order indicated around the middle \
of the first page of the output. 
\
\>", "Text",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[BoxData[{
    \(\nSetKnowns[a, Inv[a], b, Inv[b], c, Inv[c], d, Inv[d]]\n\), 
    \(SetUnknowns[{z}, {x, y, w}]\n\)}], "Input"],

Cell["\<\
Tell NCAlgebra to solve for our unknown variables
\
\>", "Text",
  Editable->False,
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[BoxData[
    \(\(\n\ \ \n\(answer = NCProcess[allRelations, 4, \ "\<DemoGBMA\>"\ ]; 
    \)\n\n\)\)], "Input"],

Cell["\<\

The spreadsheet which appears shows that, if a, b, c and d are invertible, \
then one can find x, y, z and w such that the matrices above are inverses of \
each other if and only if z b z = z + d a c.

The spreadsheet also gives formulas for x, y and w in terms of z.
\
\>", "Text",
  Editable->False]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
CellGrouping->Manual,
WindowSize->{895, 655},
WindowMargins->{{Automatic, 5}, {5, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"Magnification"->1},
StyleDefinitions -> "HTML.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 129, 6, 52, "Text"],

Cell[CellGroupData[{
Cell[1863, 59, 172, 8, 119, "H2"],
Cell[2038, 69, 208, 7, 68, "Text"],
Cell[2249, 78, 228, 4, 50, "Text"],
Cell[2480, 84, 433, 17, 62, "Text"],
Cell[2916, 103, 633, 16, 114, "Text"],
Cell[3552, 121, 402, 10, 84, "Text"],

Cell[CellGroupData[{
Cell[3979, 135, 166, 3, 59, "Input"],
Cell[4148, 140, 125, 2, 27, "Output"]
}, Open  ]],
Cell[4288, 145, 518, 16, 66, "Text"],

Cell[CellGroupData[{
Cell[4831, 165, 103, 2, 43, "Input"],
Cell[4937, 169, 50, 1, 27, "Output"],
Cell[4990, 172, 50, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5077, 178, 171, 3, 59, "Input"],
Cell[5251, 183, 117, 2, 27, "Output"],
Cell[5371, 187, 117, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5525, 194, 446, 7, 139, "Input"],
Cell[5974, 203, 188, 3, 43, "Output"],
Cell[6165, 208, 208, 3, 43, "Output"],
Cell[6376, 213, 362, 5, 59, "Output"]
}, Open  ]],
Cell[6753, 221, 254, 7, 84, "Text"],
Cell[7010, 230, 132, 2, 91, "Input"],
Cell[7145, 234, 147, 6, 66, "Text"],
Cell[7295, 242, 115, 2, 91, "Input"],
Cell[7413, 246, 312, 9, 122, "Text"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

